@@ -4,6 +4,10 @@
 #ifndef XYSSL_CERTS_H
 #define XYSSL_CERTS_H
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 extern char test_ca_crt[];
 extern char test_ca_key[];
 extern char test_ca_pwd[];
@@ -13,4 +17,8 @@ extern char test_cli_crt[];
 extern char test_cli_key[];
 extern char xyssl_ca_crt[];
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* certs.h */
