@@ -3,19 +3,19 @@
  *
  *  Copyright (C) 2006-2007  Christophe Devine
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License, version 2.1 as published by the Free Software Foundation.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- *  This library is distributed in the hope that it will be useful,
+ *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- *  MA  02110-1301  USA
+ *  You should have received a copy of the GNU General Public License along
+ *  with this program; if not, write to the Free Software Foundation, Inc.,
+ *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 /*
  *  RSA was designed by Ron Rivest, Adi Shamir and Len Adleman.
@@ -73,14 +73,12 @@ int rsa_gen_key( rsa_context *ctx, int nbits, int exponent )
      */
     MPI_CHK( mpi_lset( &ctx->E, exponent ) );
 
-    nbits >>= 1;
-
     do
     {
-        MPI_CHK( mpi_gen_prime( &ctx->P, nbits, 0, 
+        MPI_CHK( mpi_gen_prime( &ctx->P, ( nbits + 1 ) >> 1, 0, 
                                 ctx->f_rng, ctx->p_rng ) );
 
-        MPI_CHK( mpi_gen_prime( &ctx->Q, nbits, 0,
+        MPI_CHK( mpi_gen_prime( &ctx->Q, ( nbits + 1 ) >> 1, 0,
                                 ctx->f_rng, ctx->p_rng ) );
 
         if( mpi_cmp_mpi( &ctx->P, &ctx->Q ) < 0 )
@@ -90,6 +88,9 @@ int rsa_gen_key( rsa_context *ctx, int nbits, int exponent )
             continue;
 
         MPI_CHK( mpi_mul_mpi( &ctx->N, &ctx->P, &ctx->Q ) );
+        if( mpi_msb( &ctx->N ) != nbits )
+            continue;
+
         MPI_CHK( mpi_sub_int( &P1, &ctx->P, 1 ) );
         MPI_CHK( mpi_sub_int( &Q1, &ctx->Q, 1 ) );
         MPI_CHK( mpi_mul_mpi( &H, &P1, &Q1 ) );
@@ -151,29 +152,32 @@ int rsa_check_pubkey( rsa_context *ctx )
 int rsa_check_privkey( rsa_context *ctx )
 {
     int ret;
-    mpi TN, P1, Q1, H, G;
+    mpi PQ, DE, P1, Q1, H, I, G;
 
     if( ( ret = rsa_check_pubkey( ctx ) ) != 0 )
         return( ret );
 
-    mpi_init( &TN, &P1, &Q1, &H, &G, NULL );
+    mpi_init( &PQ, &DE, &P1, &Q1, &H, &I, &G, NULL );
 
-    MPI_CHK( mpi_mul_mpi( &TN, &ctx->P, &ctx->Q ) );
+    MPI_CHK( mpi_mul_mpi( &PQ, &ctx->P, &ctx->Q ) );
+    MPI_CHK( mpi_mul_mpi( &DE, &ctx->D, &ctx->E ) );
     MPI_CHK( mpi_sub_int( &P1, &ctx->P, 1 ) );
     MPI_CHK( mpi_sub_int( &Q1, &ctx->Q, 1 ) );
     MPI_CHK( mpi_mul_mpi( &H, &P1, &Q1 ) );
+    MPI_CHK( mpi_mod_mpi( &I, &DE, &H  ) );
     MPI_CHK( mpi_gcd( &G, &ctx->E, &H  ) );
 
-    if( mpi_cmp_mpi( &TN, &ctx->N ) == 0 &&
+    if( mpi_cmp_mpi( &PQ, &ctx->N ) == 0 &&
+        mpi_cmp_int( &I, 1 ) == 0 &&
         mpi_cmp_int( &G, 1 ) == 0 )
     {
-        mpi_free( &TN, &P1, &Q1, &H, &G, NULL );
+        mpi_free( &G, &I, &H, &Q1, &P1, &DE, &PQ, NULL );
         return( 0 );
     }
 
 cleanup:
 
-    mpi_free( &TN, &P1, &Q1, &H, &G, NULL );
+    mpi_free( &G, &I, &H, &Q1, &P1, &DE, &PQ, NULL );
     return( XYSSL_ERR_RSA_KEY_CHECK_FAILED | ret );
 }
 
