@@ -3,25 +3,26 @@
  *
  *  Copyright (C) 2006-2007  Christophe Devine
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License, version 2.1 as published by the Free Software Foundation.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- *  This library is distributed in the hope that it will be useful,
+ *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- *  MA  02110-1301  USA
+ *  You should have received a copy of the GNU General Public License along
+ *  with this program; if not, write to the Free Software Foundation, Inc.,
+ *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include "xyssl/config.h"
 
 #if defined(XYSSL_SSL_CLI_C)
 
+#include "xyssl/debug.h"
 #include "xyssl/ssl.h"
 
 #include <string.h>
@@ -118,6 +119,35 @@ static int ssl_write_client_hello( ssl_context *ssl )
     *p++ = 1;
     *p++ = SSL_COMPRESS_NULL;
 
+    if ( ssl->hostname != NULL ) {
+        SSL_DEBUG_MSG( 3, ( "client hello, server name extension: %s", ssl->hostname ) );
+
+        // Extensions length
+        *p++ = (unsigned char)( ( ( ssl->hostname_len + 9 ) >>  8 ) & 0xff );
+        *p++ = (unsigned char)( ( ( ssl->hostname_len + 9 )       ) & 0xff );
+
+        // Extension type
+        *p++ = (unsigned char)( ( TLS_EXT_SERVERNAME >>  8 ) & 0xff );
+        *p++ = (unsigned char)( ( TLS_EXT_SERVERNAME       ) & 0xff );
+        // Extension length
+        *p++ = (unsigned char)( ( ( ssl->hostname_len + 5 ) >>  8 ) & 0xff );
+        *p++ = (unsigned char)( ( ( ssl->hostname_len + 5 )       ) & 0xff );
+
+        // Server name list length
+        *p++ = (unsigned char)( ( ( ssl->hostname_len + 3 ) >>  8 ) & 0xff );
+        *p++ = (unsigned char)( ( ( ssl->hostname_len + 3 )       ) & 0xff );
+
+        // Name type
+        *p++ = (unsigned char)( ( TLS_EXT_SERVERNAME_HOSTNAME ) & 0xff );
+        // Hostname length
+        *p++ = (unsigned char)( ( ( ssl->hostname_len ) >>  8 ) & 0xff );
+        *p++ = (unsigned char)( ( ( ssl->hostname_len )       ) & 0xff );
+        // Hostname
+        memcpy( p, ssl->hostname, ssl->hostname_len );
+        p += ssl->hostname_len;
+
+    }
+
     ssl->out_msglen  = p - buf;
     ssl->out_msgtype = SSL_MSG_HANDSHAKE;
     ssl->out_msg[0]  = SSL_HS_CLIENT_HELLO;
@@ -139,6 +169,7 @@ static int ssl_parse_server_hello( ssl_context *ssl )
 {
     time_t t;
     int ret, i, n;
+    int ext_len;
     unsigned char *buf;
 
     SSL_DEBUG_MSG( 2, ( "=> parse server hello" ) );
@@ -201,8 +232,20 @@ static int ssl_parse_server_hello( ssl_context *ssl )
      *    39  . 38+n  session id
      *   39+n . 40+n  chosen cipher
      *   41+n . 41+n  chosen compression alg.
+     *   42+n . 43+n  extensions length
+     *   44+n . 44+n+m extensions
      */
-    if( n < 0 || n > 32 || ssl->in_hslen != 42 + n )
+    if( n < 0 || n > 32 || ssl->in_hslen > 42 + n )
+    {
+        ext_len = ( ( buf[42 + n] <<  8 )
+                  | ( buf[43 + n]       ) ) + 2;
+    }
+    else
+    {
+        ext_len = 0;
+    }
+
+    if( n < 0 || n > 32 || ssl->in_hslen != 42 + n + ext_len )
     {
         SSL_DEBUG_MSG( 1, ( "bad server hello message" ) );
         return( XYSSL_ERR_SSL_BAD_HS_SERVER_HELLO );
@@ -259,6 +302,8 @@ static int ssl_parse_server_hello( ssl_context *ssl )
         return( XYSSL_ERR_SSL_BAD_HS_SERVER_HELLO );
     }
 
+    // TODO: Process extensions
+
     SSL_DEBUG_MSG( 2, ( "<= parse server hello" ) );
 
     return( 0 );
