@@ -3,19 +3,19 @@
  *
  *  Copyright (C) 2006-2007  Christophe Devine
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License, version 2.1 as published by the Free Software Foundation.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- *  This library is distributed in the hope that it will be useful,
+ *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- *  MA  02110-1301  USA
+ *  You should have received a copy of the GNU General Public License along
+ *  with this program; if not, write to the Free Software Foundation, Inc.,
+ *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 /*
  *  The SSL 3.0 specification was drafted by Netscape in 1996,
@@ -33,6 +33,7 @@
 #include "xyssl/aes.h"
 #include "xyssl/arc4.h"
 #include "xyssl/des.h"
+#include "xyssl/debug.h"
 #include "xyssl/ssl.h"
 
 #include <string.h>
@@ -240,6 +241,11 @@ int ssl_derive_keys( ssl_context *ssl )
 #endif
 
 #if defined(XYSSL_AES_C)
+        case SSL_RSA_AES_128_SHA:
+            ssl->keylen = 16; ssl->minlen = 32;
+            ssl->ivlen  = 16; ssl->maclen = 20;
+            break;
+
         case SSL_RSA_AES_256_SHA:
         case SSL_EDH_RSA_AES_256_SHA:
             ssl->keylen = 32; ssl->minlen = 32;
@@ -303,6 +309,11 @@ int ssl_derive_keys( ssl_context *ssl )
 #endif
 
 #if defined(XYSSL_AES_C)
+        case SSL_RSA_AES_128_SHA:
+            aes_setkey_enc( (aes_context *) ssl->ctx_enc, key1, 128 );
+            aes_setkey_dec( (aes_context *) ssl->ctx_dec, key2, 128 );
+            break;
+
         case SSL_RSA_AES_256_SHA:
         case SSL_EDH_RSA_AES_256_SHA:
             aes_setkey_enc( (aes_context *) ssl->ctx_enc, key1, 256 );
@@ -1558,6 +1569,9 @@ int ssl_init( ssl_context *ssl )
     memset( ssl-> in_ctr, 0, SSL_BUFFER_LEN );
     memset( ssl->out_ctr, 0, SSL_BUFFER_LEN );
 
+    ssl->hostname = NULL;
+    ssl->hostname_len = 0;
+
      md5_starts( &ssl->fin_md5  );
     sha1_starts( &ssl->fin_sha1 );
 
@@ -1567,11 +1581,6 @@ int ssl_init( ssl_context *ssl )
 /*
  * SSL set accessors
  */
-void ssl_set_debuglvl( ssl_context *ssl, int debuglvl )
-{
-    ssl->debuglvl   = debuglvl;
-}
-
 void ssl_set_endpoint( ssl_context *ssl, int endpoint )
 {
     ssl->endpoint   = endpoint;
@@ -1582,19 +1591,28 @@ void ssl_set_authmode( ssl_context *ssl, int authmode )
     ssl->authmode   = authmode;
 }
 
-void ssl_set_rng( ssl_context *ssl, int (*f_rng)(void *), void *p_rng )
+void ssl_set_rng( ssl_context *ssl,
+                  int (*f_rng)(void *),
+                  void *p_rng )
 {
     ssl->f_rng      = f_rng;
     ssl->p_rng      = p_rng;
 }
 
+void ssl_set_dbg( ssl_context *ssl,
+                  void (*f_dbg)(void *, int, char *),
+                  void  *p_dbg )
+{
+    ssl->f_dbg      = f_dbg;
+    ssl->p_dbg      = p_dbg;
+}
+
 void ssl_set_bio( ssl_context *ssl,
             int (*f_recv)(void *, unsigned char *, int), void *p_recv,
             int (*f_send)(void *, unsigned char *, int), void *p_send )
 {
     ssl->f_recv     = f_recv;
     ssl->f_send     = f_send;
-
     ssl->p_recv     = p_recv;
     ssl->p_send     = p_send;
 }
@@ -1653,6 +1671,20 @@ int ssl_set_dh_param( ssl_context *ssl, char *dhm_P, char *dhm_G )
     return( 0 );
 }
 
+int ssl_set_hostname( ssl_context *ssl, char *hostname )
+{
+    if( hostname == NULL )
+        return( XYSSL_ERR_SSL_BAD_INPUT_DATA );
+
+    ssl->hostname_len = strlen( hostname );
+    ssl->hostname = (unsigned char *) malloc( ssl->hostname_len );
+
+    memcpy( ssl->hostname, (unsigned char *) hostname,
+            ssl->hostname_len );
+
+    return( 0 );
+}
+
 /*
  * SSL get accessors
  */
@@ -1687,6 +1719,9 @@ char *ssl_get_cipher( ssl_context *ssl )
 #endif
 
 #if defined(XYSSL_AES_C)
+        case SSL_RSA_AES_128_SHA:
+            return( "SSL_RSA_AES_128_SHA" );
+
         case SSL_RSA_AES_256_SHA:
             return( "SSL_RSA_AES_256_SHA" );
 
@@ -1713,6 +1748,7 @@ int ssl_default_ciphers[] =
 #endif
 
 #if defined(XYSSL_AES_C)
+    SSL_RSA_AES_128_SHA,
     SSL_RSA_AES_256_SHA,
 #endif
 #if defined(XYSSL_DES_C)
@@ -1833,23 +1869,28 @@ int ssl_write( ssl_context *ssl, unsigned char *buf, int len )
         }
     }
 
-    if( ( ret = ssl_flush_output( ssl ) ) != 0 )
-    {
-        SSL_DEBUG_RET( 1, "ssl_flush_output", ret );
-        return( ret );
-    }
-
     n = ( len < SSL_MAX_CONTENT_LEN )
         ? len : SSL_MAX_CONTENT_LEN;
 
-    ssl->out_msglen  = n;
-    ssl->out_msgtype = SSL_MSG_APPLICATION_DATA;
-    memcpy( ssl->out_msg, buf, n );
-
-    if( ( ret = ssl_write_record( ssl ) ) != 0 )
+    if( ssl->out_left != 0 )
     {
-        SSL_DEBUG_RET( 1, "ssl_write_record", ret );
-        return( ret );
+        if( ( ret = ssl_flush_output( ssl ) ) != 0 )
+        {
+            SSL_DEBUG_RET( 1, "ssl_flush_output", ret );
+            return( ret );
+        }
+    }
+    else
+    {
+        ssl->out_msglen  = n;
+        ssl->out_msgtype = SSL_MSG_APPLICATION_DATA;
+        memcpy( ssl->out_msg, buf, n );
+
+        if( ( ret = ssl_write_record( ssl ) ) != 0 )
+        {
+            SSL_DEBUG_RET( 1, "ssl_write_record", ret );
+            return( ret );
+        }
     }
 
     SSL_DEBUG_MSG( 2, ( "<= write" ) );
@@ -1921,6 +1962,13 @@ void ssl_free( ssl_context *ssl )
     dhm_free( &ssl->dhm_ctx );
 #endif
 
+    if ( ssl->hostname != NULL)
+    {
+        memset( ssl->hostname, 0, ssl->hostname_len );
+        free( ssl->hostname );
+        ssl->hostname_len = 0;
+    }
+
     memset( ssl, 0, sizeof( ssl_context ) );
 
     SSL_DEBUG_MSG( 2, ( "<= free" ) );
