@@ -3,19 +3,19 @@
  *
  *  Copyright (C) 2006-2007  Christophe Devine
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License, version 2.1 as published by the Free Software Foundation.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- *  This library is distributed in the hope that it will be useful,
+ *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- *  MA  02110-1301  USA
+ *  You should have received a copy of the GNU General Public License along
+ *  with this program; if not, write to the Free Software Foundation, Inc.,
+ *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 /*
  *  The ITU-T X.509 standard defines a certificat format for PKI.
@@ -608,7 +608,16 @@ static int x509_get_ext( unsigned char **p,
             continue;
 
         if( ( ret = asn1_get_bool( p, end2, &is_cacert ) ) != 0 )
-            return( XYSSL_ERR_X509_CERT_INVALID_EXTENSIONS | ret );
+        {
+            if( ret == XYSSL_ERR_ASN1_UNEXPECTED_TAG )
+                ret = asn1_get_int( p, end2, &is_cacert );
+
+            if( ret != 0 )
+                return( XYSSL_ERR_X509_CERT_INVALID_EXTENSIONS | ret );
+
+            if( is_cacert != 0 )
+                is_cacert  = 1;
+        }
 
         if( *p == end2 )
             continue;
@@ -690,7 +699,11 @@ int x509parse_crt( x509_cert *chain, unsigned char *buf, int buflen )
         s2 += 25;
         if( *s2 == '\r' ) s2++;
         if( *s2 == '\n' ) s2++;
-            else return( XYSSL_ERR_X509_CERT_INVALID_PEM );
+        else
+        {
+            free( p );
+            return( XYSSL_ERR_X509_CERT_INVALID_PEM );
+        }
 
         buflen -= s2 - buf;
         buf = s2;
@@ -1386,15 +1399,11 @@ int x509parse_dn_gets( char *buf, char *end, x509_name *dn )
 char *x509parse_cert_info( char *prefix, x509_cert *crt )
 {
     int i, n;
-    char *buf, *p, *end;
-
-    if( ( buf = (char *) malloc( 4096 ) ) == NULL )
-        return( NULL );
-
-    memset( buf, 0, 4096 );
+    char *p, *end;
+    static char buf[512];
 
     p = buf;
-    end = buf + 4096 - 1;
+    end = buf + sizeof( buf ) - 1;
 
     p += snprintf( p, end - p, "%scert. version : %d\n",
                                prefix, crt->version );
@@ -1404,7 +1413,7 @@ char *x509parse_cert_info( char *prefix, x509_cert *crt )
     n = ( crt->serial.len <= 32 )
         ? crt->serial.len  : 32;
 
-    for( i = 1; i < n; i++ )
+    for( i = 0; i < n; i++ )
         p += snprintf( p, end - p, "%02X%s",
                 crt->serial.p[i], ( i < n - 1 ) ? ":" : "" );
 
@@ -1495,6 +1504,7 @@ int x509parse_verify( x509_cert *crt,
                       x509_cert *trust_ca,
                       char *cn, int *flags )
 {
+    int cn_len;
     int hash_id;
     int pathlen;
     x509_cert *cur;
@@ -1506,11 +1516,13 @@ int x509parse_verify( x509_cert *crt,
     if( cn != NULL )
     {
         name = &crt->subject;
+        cn_len = strlen( cn );
 
         while( name != NULL )
         {
-            if( memcmp( name->oid.p, "\x55\x04\x03", 3 ) == 0 &&
-                memcmp( name->val.p, cn, strlen( cn )  ) == 0 )
+            if( memcmp( name->oid.p, OID_CN,  3 ) == 0 &&
+                memcmp( name->val.p, cn, cn_len ) == 0 &&
+                name->val.len == cn_len )
                 break;
 
             name = name->next;
@@ -1585,6 +1597,8 @@ int x509parse_verify( x509_cert *crt,
             *flags &= ~BADCERT_NOT_TRUSTED;
             break;
         }
+
+        trust_ca = trust_ca->next;
     }
 
     if( *flags != 0 )
@@ -1615,6 +1629,7 @@ void x509_free( x509_cert *crt )
         {
             name_prv = name_cur;
             name_cur = name_cur->next;
+            memset( name_prv, 0, sizeof( x509_name ) );
             free( name_prv );
         }
 
@@ -1623,11 +1638,15 @@ void x509_free( x509_cert *crt )
         {
             name_prv = name_cur;
             name_cur = name_cur->next;
+            memset( name_prv, 0, sizeof( x509_name ) );
             free( name_prv );
         }
 
         if( cert_cur->raw.p != NULL )
+        {
+            memset( cert_cur->raw.p, 0, cert_cur->raw.len );
             free( cert_cur->raw.p );
+        }
 
         cert_cur = cert_cur->next;
     }
@@ -1639,6 +1658,7 @@ void x509_free( x509_cert *crt )
         cert_prv = cert_cur;
         cert_cur = cert_cur->next;
 
+        memset( cert_prv, 0, sizeof( x509_cert ) );
         if( cert_prv != crt )
             free( cert_prv );
     }
